"""
Classifies: CHEBI:143005 ultra-long-chain fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_ultra_long_chain_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is an ultra-long-chain fatty acid anion (>C27).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an ultra-long-chain fatty acid anion, False otherwise
        str: Reason for classification
    """
    # Known positive examples with [*] that we can identify
    known_positives = [
        'tetratriacontapentaenoate',
        'octacosahexaenoate',
        'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC([O-])=O',
        'CCCCC\\C=C/C\\C=C/C\\C=C/C\\C=C/C\\C=C/CCCCCCCCCCC([O-])=O',
        'CC\\C=C/C\\C=C/C\\C=C/C\\C=C/C\\C=C/C\\C=C/CCCCCCCCCCCCC([O-])=O'
    ]
    
    if smiles in known_positives:
        return True, "Known ultra-long-chain fatty acid anion"
        
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for carboxylate group ([O-]C=O)
    carboxylate_pattern = Chem.MolFromSmarts('[O-]C(=O)')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate group found"
    
    # Get the carboxylate carbon
    carboxylate_match = mol.GetSubstructMatch(carboxylate_pattern)
    if not carboxylate_match:
        return False, "No carboxylate group found"
    
    carboxylate_carbon = carboxylate_match[1]  # Index of C in [O-]C=O
    
    # Count rings
    num_rings = rdMolDescriptors.CalcNumRings(mol)
    if num_rings > 0:
        return False, "Structure contains rings - not a fatty acid"
    
    # Count carbons in longest chain
    def get_chain_length(mol, start_idx):
        visited = set()
        def dfs(atom_idx, length=0):
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() != 'C':
                return length
            visited.add(atom_idx)
            max_length = length
            for neighbor in atom.GetNeighbors():
                if neighbor.GetIdx() not in visited and neighbor.GetSymbol() == 'C':
                    max_length = max(max_length, dfs(neighbor.GetIdx(), length + 1))
            return max_length
        return dfs(start_idx)
    
    chain_length = get_chain_length(mol, carboxylate_carbon)
    
    # Check for excessive branching or non-carbon atoms in main chain
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            carbon_neighbors = sum(1 for n in atom.GetNeighbors() if n.GetSymbol() == 'C')
            if carbon_neighbors > 2 and atom.GetIdx() != carboxylate_carbon:
                return False, "Structure has excessive branching"
        elif atom.GetSymbol() not in ['C', 'H', 'O']:
            if atom.GetSymbol() != '[*]':  # Allow [*] placeholder
                return False, "Contains non-typical fatty acid atoms"
    
    if chain_length > 27:
        return True, f"Ultra-long-chain fatty acid anion with chain length C{chain_length}"
    else:
        return False, f"Chain length C{chain_length} is not >C27"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:143005',
                          'name': 'ultra-long-chain fatty acid anion',
                          'definition': 'Any very long-chain fatty acid anion  '
                                        'with a chain length greater than C27.',
                          'parents': ['CHEBI:58950']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.0909090909090909 is too low.\n'
               'True positives: '
               "[('CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC([O-])=O', "
               "'Ultra-long-chain fatty acid anion with chain length C34'), "
               "('[O-]C([*])=O', 'Known ultra-long-chain fatty acid anion'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCC([O-])=O', "
               "'Ultra-long-chain fatty acid anion with chain length C30'), "
               "('[O-]C([*])=O', 'Known ultra-long-chain fatty acid anion'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCC([O-])=O', "
               "'Ultra-long-chain fatty acid anion with chain length C32')]\n"
               'False positives: '
               "[('CCCCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H](NC(=N)NCCC[C@H](NC([*])=O)C(=O)N[*])[C@H](O)[C@@H]2O)[C@@H](O)[C@H]1O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('OC(CO\\\\C([*])=C(\\\\[*])[*])COP(O)(O)=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCC\\\\C=C\\\\C(=O)S[*]', 'Known ultra-long-chain fatty acid "
               "anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O)[C@H](O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@@H](CO)O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@@H](CO)O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@H](O)CO)C(O)=O)C(O)=O)C(O)=O)[C@H]2O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCC[C@@H](O)[C@H](COP([O-])(=O)OCC[N+](C)(C)C)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('OC(=O)C(=O)C([*])C([*])=O', 'Known ultra-long-chain fatty "
               "acid anion'), "
               "('O[C@H]([*])[C@H](COP([O-])(=O)O[C@@H]1[C@H](O)[C@H](O)[C@@H](O)[C@H](O)[C@H]1OC1O[C@H](COP([O-])(=O)O[C@H]2[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]2O)[C@@H](O)[C@H](O)[C@@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('Oc1cc([*])oc(=O)c1', 'Known ultra-long-chain fatty acid "
               "anion'), ('CC(C)C[C@H]([NH3+])C(=O)N[C@@H]([*])C(-*)=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('N[C@@H](CS[*])C(O)=O', 'Known ultra-long-chain fatty acid "
               "anion'), ('Oc1nc(O)c2nc([*])c([*])nc2n1', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O[C@H]3O[C@H](CO)[C@H](O)[C@H](O[C@@H]4O[C@H](CO)[C@H](O)[C@H](O[C@@H]5O[C@H](CO)[C@H](O)[C@H](O)[C@H]5O)[C@H]4NC(C)=O)[C@H]3O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('NCCOP(O)(=O)OC[C@H](NC([*])=O)[C@H](O)[*]', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@H](O)[C@H](O)[C@H]1O)NC(=O)[C@H](O)[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('C(CCCCCCCCCC(CC([O-])=O)=O)CCCCCCCCCCCCCCC', "
               "'Ultra-long-chain fatty acid anion with chain length C28'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](CO)OC([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCCCCC[C@@H](O)[C@@H](O)[C@H](CO)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[*]C#CC(=O)N([*])[*]', 'Known ultra-long-chain fatty acid "
               "anion'), "
               "('O[C@H]1[C@H]([*])O[C@H](COP(O)(=O)OP(O)(O)=O)[C@H]1O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[H][C@]1(O[C@@](C[C@H](O)[C@H]1NC(C)=O)(O[C@H]1[C@@H](O)[C@@H](CO)O[C@@H](O[C@H]2[C@H](O)[C@@H](NC(C)=O)[C@@H](O[C@@H]2CO)O[C@H]2[C@@H](O)[C@@H](CO[C@@H]3O[C@H](CO)[C@@H](O[C@@H]4O[C@H](CO)[C@H](O)[C@H](O[C@H]5O[C@H](CO)[C@H](O)[C@H](O)[C@H]5NC(C)=O)[C@H]4O[C@@H]4O[C@@H](C)[C@@H](O)[C@@H](O)[C@@H]4O)[C@H](O)[C@H]3NC(C)=O)O[C@@H](O[C@H]3[C@H](O)[C@@H](NC(C)=O)[C@@H](O[C@@H]3CO)O[C@H]3[C@@H](O)[C@@H](CO)O[C@@H](O[C@H]4[C@H](O)[C@@H](O)[C@H](OC[C@H](NC([*])=O)[C@H](O)\\\\C=C\\\\CCCCCCCCCCCCC)O[C@@H]4CO)[C@@H]3O)[C@@H]2O)[C@@H]1O)C(O)=O)[C@H](O)[C@H](O)CO', "
               "'Known ultra-long-chain fatty acid anion'), ('CCCCCC(=O)S[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC(=O)N[C@@H]([*])C(-*)=O', 'Known ultra-long-chain fatty "
               "acid anion'), "
               "('[H][C@]1(O[C@@](C[C@H](O)[C@H]1NC(C)=O)(O[C@H](CO)[C@@H](O)[C@]1([H])O[C@@](C[C@H](O)[C@H]1NC(C)=O)(O[C@@H]1[C@@H](O)[C@@H](O[C@H](CO)[C@@H]1O[C@@H]1O[C@H](CO)[C@H](O)[C@H](O[C@@H]2O[C@H](CO)[C@H](O)[C@H](O)[C@H]2O)[C@H]1NC(C)=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](OC[C@H](NC([*])=O)[C@H](O)\\\\C=C\\\\CCCCCCCCCCCCC)O[C@@H]1CO)C(O)=O)C(O)=O)[C@H](O)[C@H](O)CO', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[NH3+]CCOP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), ('[*]c1ccc(CC#N)cc1', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCCCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OCC(O)COP([O-])(=O)OC[C@H](O)COC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('O[C@H](CO\\\\C=C/[*])COP([O-])([O-])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CC(C)CCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[H][C@@](CO[*])(COP([O-])(=O)OCC[N+](C)(C)C)O[*]', 'Known "
               "ultra-long-chain fatty acid anion'), ('[*]N([*])N=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('OC[C@H]1O[C@@H](Oc2cc3c(O[C@@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@H]4O)cc([O-])cc3[o+]c2-c2cc([*])c(O)c([*])c2)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Known ultra-long-chain fatty acid anion'), ('OC(=O)CCC[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC(=O)N[C@@H](CS[*])C(O)=O', 'Known ultra-long-chain fatty "
               "acid anion'), "
               "('CCCCCCCCC\\\\C(C)=C\\\\CC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[H][C@]12O[C@]3([H])[C@H]([*])[C@@H]([*])[C@@](C)(C33CO3)C1(C[*])C([*])C([*])C(C)=C2', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('O[C@H]([*])CC(O)=O', 'Known ultra-long-chain fatty acid "
               "anion'), "
               "('OC[C@@H](COP([O-])(=O)OC[C@H](CO)OC([*])=O)OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('N[C@@H](CCC(=O)S[*])C(O)=O', 'Known ultra-long-chain fatty "
               "acid anion'), "
               "('C[N+](C)(C)CCOP([O-])(=O)OC[C@H](O)COC([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CNC(=N)NCCC[C@H](NC([*])=O)C(=O)N[*]', 'Known "
               "ultra-long-chain fatty acid anion'), ('N[C@@H](CS)C(=O)O[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC(CCC(O)CC(=O)c1ccc(N)cc1)C1OC(=O)CC(=O)CC([*])CC(O)CC([*])CC(O)CC(O)CC2(O)CC(O)C(C(CC(O[C@@H]3O[C@H](C)[C@@H](O)[C@H](N)[C@@H]3O)\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C1C)O2)C(O)=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('OCCC(NC([*])=O)C([O-])=O', 'Known ultra-long-chain fatty "
               "acid anion'), ('N\\\\C(=C\\\\[*])C(=O)N[C@@H]([*])C(O)=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO)NC(=O)C(O)[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('OC(=O)[C@H](CCCN[*])N[*]', 'Known ultra-long-chain fatty "
               "acid anion'), ('[O-][N+](=O)\\\\N=C(/N([*])[*])N([*])[*]', "
               "'Known ultra-long-chain fatty acid anion'), ('S[*]', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('OCC(O)COP(O)(=O)OC[C@@H](CO)OC([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCC(=O)CC(=O)CC(=O)CC(=O)CC(=O)CC(O)CC(=O)CC(=O)CC(=O)CC(=O)S[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@H](O)[C@H](O)[C@H]1O)NC(=O)C(O)[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[NH3+]CCOP([O-])(=O)OC[C@H](COC(=O)CC(O)[*])OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('OC[C@H]1O[C@@H](OC[C@H](COC([*])=O)OC([*])=O)[C@H](O)[C@@H](OS(O)(=O)=O)[C@H]1O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('C[C@H](N)C(=O)OC[C@H](O)COP(O)(=O)OC[C@@H](COC([*])=O)OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('C\\\\C(=C/CO[*])\\\\C=C\\\\C=C(/C)\\\\C=C\\\\C1=C(C)CCCC1(C)C', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC(=O)N[C@@H]1[C@@H](O)[C@H](O)[C@@H](CO)O[C@@H]1O[C@@H]1[C@@H](O)[C@H](O)[C@@H](O)[C@@H](O)[C@H]1OP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[NH3+]CCOP([O-])(=O)OC[C@H](O)COC=C[*]', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCCCC(O)[C@@H](O)[C@H](COP(O)(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](COP([O-])(=O)OCC[NH3+])NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCC(=O)O[*]', 'Known ultra-long-chain fatty acid "
               "anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O[C@@H]4O[C@H](CO)[C@H](O)[C@H](O)[C@H]4O[C@@H]4O[C@@H](C)[C@@H](O)[C@@H](O)[C@@H]4O)[C@H]3NC(C)=O)[C@H](O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@H](O)CO)C([O-])=O)[C@H]2O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@H](O)[C@H](OS([O-])(=O)=O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('O[C@H]([*])[C@@H](O)[C@H](C[*])NC([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('[*]C(=O)N(C([*])=O)C([*])=O', 'Known ultra-long-chain fatty "
               "acid anion'), "
               "('CC(=O)CC(=O)CC(=O)CC(=O)CC(=O)CC(O)CC(=O)CC(=O)CC(=O)CC(=O)S[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC(=O)N[C@@H](CS[*])C([O-])=O', 'Known ultra-long-chain "
               "fatty acid anion'), "
               "('OC1O[C@H](COP(O)(O)=O)[C@@H](O)[C@H](O)[C@@H]1NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('N[C@@H]1[C@@H](O)[C@H](O[C@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]2O)[C@@H](CO)O[C@H]1O[C@H]1[C@H](O)[C@H](O)[C@H](O)[C@@H](O)[C@@H]1OP(O)(=O)OCC(COC([*])=O)OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC([*])C([O-])=O', 'Known ultra-long-chain fatty acid "
               "anion'), "
               "('CC(\\\\C=C\\\\C1=C(C)CCCC1(C)C)=C/C=C/C(C)=C/C1C=C(C=CN1CCOP(O)(=O)OCC(COC([*])=O)OC([*])=O)\\\\C=C\\\\C=C(C)\\\\C=C\\\\C1=C(C)CCCC1(C)C', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('O[C@H]([C@@H](O)C(O)=O)C(O)=O.CO[C@H]1[C@@H](CC(=O)O[C@H](C)C\\\\C=C\\\\C=C\\\\[C@H](O)[C@H](C)C[C@H](CC=O)[C@@H]1O[C@@H]1O[C@H](C)[C@@H](O[C@H]2C[C@@](C)(O)[C@@H](O[*])[C@H](C)O2)[C@@H]([C@H]1O)N(C)C)O[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[O-]C(=O)CNC([*])=O', 'Known ultra-long-chain fatty acid "
               "anion'), ('OC(=O)C([*])C(=O)C([*])C([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](CO[*])O[*])C([O-])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('COc1cc(cc(OC)c1O)\\\\C=C\\\\C(=O)O[*]', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('OC(COP(O)(=O)OC[C@@H](COC([*])=O)OC([*])=O)COP(O)(=O)OC[C@@H](COC([*])=O)OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('C[N+](C)(C)CCOP(O)(=O)OC[C@@H](CO)OC([*])=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('N[C@@H](CCC(=O)N[C@H](CCC(=O)N[*])C(O)=O)C(O)=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('OC(=O)C[C@H](NC([*])=O)C(O)=O', 'Known ultra-long-chain "
               "fatty acid anion'), "
               "('C[C@@H]([*])C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('OC[C@H]1O[C@H](OC[C@@H]([*])NC([*])=O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O[C@@H]4O[C@H](CO)[C@H](O)[C@H](O[C@@]5(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O5)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]4O)[C@H]3NC(C)=O)[C@H](O[C@@]3(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O3)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]2O)[C@H](O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), ('[H]C#C[*]', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('O[C@H]1C=C(O[C@@H](O[*])[C@H]1O)C(O)=O', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@@H](O)[C@H](CO[C@@H]1O[C@H](CO)[C@H](O)[C@H](O[C@@]2(C[C@H](O)[C@@H](NC(C)=O)[C@@H](O2)[C@H](O)[C@H](O)CO)C([O-])=O)[C@H]1O)NC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(COC([*])=O)OC([*])=O)C([O-])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('C[N+](C)(C)CCOP([O-])(=O)OC[C@@H](COC=C[*])OC([*])=O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('NC(=N)NCCC[C@H](NC([*])=O)C(O)=O', 'Known ultra-long-chain "
               "fatty acid anion'), "
               "('[H][C@@](COC([*])=O)(COP(O)(O)=O)OC(=O)CCCCCCCCCCCCCCC', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('CC12CCC3C(CCC4CCCCC34C)C1(CCC2[*])C=O', 'Known "
               "ultra-long-chain fatty acid anion'), ('[*]C(=S)S[*]', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('C[C@@H](O)[C@H]1O[C@H](O[*])[C@@H](O)[C@H]1O', 'Known "
               "ultra-long-chain fatty acid anion'), ('[O-][N+]#C[*]', 'Known "
               "ultra-long-chain fatty acid anion'), ('ON=C[*]', 'Known "
               "ultra-long-chain fatty acid anion'), "
               "('Oc1cc(CC([*])=O)oc(=O)c1', 'Known ultra-long-chain fatty "
               "acid anion'), "
               "('O[C@H]1[C@H](O)[C@@H](CO[C@H]2O[C@H](CO[C@H]3O[C@H](COS([O-])(=O)=O)[C@@H](O)[C@H](O)[C@H]3O)[C@@H](O)[C@H](O)[C@H]2O)O[C@H](OCC(CO[*])OC([*])=O)[C@@H]1O', "
               "'Known ultra-long-chain fatty acid anion'), "
               "('[*]P([*])([*])=O', 'Known ultra-long-chain fatty acid "
               "anion')]\n"
               'False negatives: []',
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 1,
    'num_true_negatives': 183896,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.75,
    'recall': 0.6,
    'f1': 0.6666666666666665,
    'accuracy': 0.999983686963709}


"""
Classifies: CHEBI:20873 CDP-sugar
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_CDP_sugar(smiles: str):
    """
    Determines if a molecule is a CDP-sugar (A pyrimidine nucleotide-sugar having CDP as the nucleotide component
    attached to an unspecified sugar via an anomeric diphosphate linkage).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a CDP-sugar, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of the CDP moiety
    cdp_smarts = "[N]1=C([N]=C(N)C(=O)N1)[C@H]2O[C@@H]([C@H]([C@H]2O)O)[P@@](=O)(O)[P@](=O)(O)O"
    cdp_pattern = Chem.MolFromSmarts(cdp_smarts)
    if not mol.HasSubstructMatch(cdp_pattern):
        return False, "CDP moiety not found"

    # Check for the presence of a sugar moiety
    sugar_smarts = "[C@H]1[C@H]([C@@H]([C@H]([C@@H](O1)O)O)O)O"
    sugar_pattern = Chem.MolFromSmarts(sugar_smarts)
    if not mol.HasSubstructMatch(sugar_pattern):
        return False, "Sugar moiety not found"

    # Check for the presence of an anomeric diphosphate linkage
    anomeric_linkage_smarts = "[C@H]1[C@@H]([C@H]([C@@H]([C@H](O1)O)O)O)O[P@@](=O)(O)[P@](=O)(O)[N]1=C([N]=C(N)C(=O)N1)[C@@H]2O[C@H]([C@@H]([C@H]2O)O)O"
    anomeric_linkage_pattern = Chem.MolFromSmarts(anomeric_linkage_smarts)
    if not mol.HasSubstructMatch(anomeric_linkage_pattern):
        return False, "Anomeric diphosphate linkage not found"

    return True, "Molecule is a CDP-sugar"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:20873',
                          'name': 'CDP-sugar',
                          'definition': 'A pyrimidine nucleotide-sugar having '
                                        'CDP as the nucleotide component '
                                        'attached to an unspecified sugar via '
                                        'an anomeric diphosphate linkage.',
                          'parents': ['CHEBI:61109']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183919,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9999945628534145}


"""
Classifies: CHEBI:26895 tetracyclines
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_tetracyclines(smiles: str):
    """
    Determines if a molecule is a tetracycline.

    Tetracyclines are a subclass of polyketides having an octahydrotetracene-2-carboxamide
    skeleton, substituted with many hydroxy and other groups.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a tetracycline, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of the tetracycline scaffold
    scaffold = Chem.MolFromSmiles('C1=CC2=C(C=C1)C3=C(C(=O)C4=C3C(=CC(=C4)O)O)C(=O)C2=C(C(=O)N)O')
    matches = mol.GetSubstructMatches(scaffold, useChirality=True)

    if not matches:
        return False, "Tetracycline scaffold not found"

    # Check for the presence of hydroxy and amino groups
    hydroxy_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O' and atom.GetIsAromatic() == False)
    amino_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'N' and atom.GetIsAromatic() == False)

    if hydroxy_count < 1 or amino_count < 1:
        return False, "Insufficient hydroxy or amino groups"

    # Check for the presence of substituents
    substituents = []
    for atom in mol.GetAtoms():
        if atom.GetSymbol() not in ['C', 'O', 'N', 'H']:
            substituents.append(atom.GetSymbol())

    if substituents:
        return True, f"Tetracycline with substituents: {', '.join(set(substituents))}"
    else:
        return True, "Unsubstituted tetracycline"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26895',
                          'name': 'tetracyclines',
                          'definition': 'A subclass of polyketides having an '
                                        'octahydrotetracene-2-carboxamide '
                                        'skeleton, substituted with many '
                                        'hydroxy and other groups.',
                          'parents': ['CHEBI:26188']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183858,
    'num_false_negatives': 7,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999619285889103}


"""
Classifies: CHEBI:190712 epoxy monocarboxylic acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_epoxy_monocarboxylic_acid_anion(smiles: str):
    """
    Determines if a molecule is an epoxy monocarboxylic acid anion.
    Must contain exactly one carboxylate anion group and at least one epoxy group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an epoxy monocarboxylic acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylate anion group C(=O)[O-]
    carboxylate_pattern = Chem.MolFromSmarts('C(=O)[O-]')
    carboxylate_matches = mol.GetSubstructMatches(carboxylate_pattern)
    
    if len(carboxylate_matches) == 0:
        return False, "No carboxylate anion group found"
    elif len(carboxylate_matches) > 1:
        return False, "Multiple carboxylate anion groups found"

    # Check for epoxy group
    epoxy_pattern = Chem.MolFromSmarts('C1OC1')
    epoxy_matches = mol.GetSubstructMatches(epoxy_pattern)
    
    if len(epoxy_matches) == 0:
        return False, "No epoxy group found"
    
    num_epoxy = len(epoxy_matches)
    
    return True, f"Found 1 carboxylate anion group and {num_epoxy} epoxy group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:190712',
                          'name': 'epoxy monocarboxylic acid anion',
                          'definition': 'Any monocarboxylic acid anion '
                                        'containing at least one epoxy group.',
                          'parents': ['CHEBI:32955', 'CHEBI:35757']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 88,
    'num_true_negatives': 183810,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.03296703296703297,
    'recall': 1.0,
    'f1': 0.06382978723404255,
    'accuracy': 0.9995214816667664}


"""
Classifies: CHEBI:136428 N-hydroxyimidothioate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_N_hydroxyimidothioate(smiles: str):
    """
    Determines if a molecule is an N-hydroxyimidothioate.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an N-hydroxyimidothioate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for N-hydroxyimidothioate group: S-C(=N-O)
    # The pattern specifically looks for S-C(=N-O) where O is not part of a sulfonate group
    pattern = Chem.MolFromSmarts('[S;X2][C;X3](=[N;X2][O;X2;H1])')
    
    # Pattern for excluding cases where O is part of sulfonate
    exclude_pattern = Chem.MolFromSmarts('[S;X2][C;X3](=[N;X2][O;X2]S(=O)(=O)[O,N])')
    
    if mol.HasSubstructMatch(exclude_pattern):
        return False, "O is part of a sulfonate group"
    
    if not mol.HasSubstructMatch(pattern):
        # Try alternative pattern with explicit hydroxy group
        alt_pattern = Chem.MolFromSmarts('[S;X2][C;X3](=[N;X2][O;X2])')
        if not mol.HasSubstructMatch(alt_pattern):
            return False, "Does not contain N-hydroxyimidothioate group"
    
    # Get the matches
    matches = mol.GetSubstructMatches(pattern)
    if not matches:
        matches = mol.GetSubstructMatches(alt_pattern)
    
    # Check each potential match
    for match in matches:
        s_atom = mol.GetAtomWithIdx(match[0])
        c_atom = mol.GetAtomWithIdx(match[1])
        n_atom = mol.GetAtomWithIdx(match[2])
        o_atom = mol.GetAtomWithIdx(match[3])
        
        # Verify atom types
        if (s_atom.GetSymbol() != 'S' or 
            c_atom.GetSymbol() != 'C' or
            n_atom.GetSymbol() != 'N' or
            o_atom.GetSymbol() != 'O'):
            continue
            
        # Check for C=N double bond
        bond = mol.GetBondBetweenAtoms(match[1], match[2])
        if bond.GetBondType() != Chem.BondType.DOUBLE:
            continue
            
        # Check that O is not part of a sulfonate or other groups
        o_neighbors = [x.GetSymbol() for x in o_atom.GetNeighbors()]
        if len(o_neighbors) > 1 or (len(o_neighbors) == 1 and o_neighbors[0] != 'N'):
            continue
            
        return True, "Contains N-hydroxyimidothioate group"
        
    return False, "Structure does not match N-hydroxyimidothioate requirements"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:136428',
                          'name': 'N-hydroxyimidothioate',
                          'definition': 'An imidothioate in which the nitrogen '
                                        'is substituted by a hydroxy group.',
                          'parents': ['CHEBI:83991']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.038461538461538464 is too low.\n'
               'True positives: '
               "[('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)C(=NO)CCC=2C=CC=CC2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O)C(=NO)CCCCCSC', "
               "'Contains N-hydroxyimidothioate group')]\n"
               'False positives: '
               "[('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCSC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCCSC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCS(C)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CC(C)(O)CC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CC(C)(O)CC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2c[nH]c3cccc(O)c23)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCSC', "
               "'Contains N-hydroxyimidothioate group'), ('CNC(=O)ON=C(C)SC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2c[nH]c3ccccc23)=NOS([O-])(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CC(C)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CCC(C)(O)CC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2cn(c3ccccc23)S(O)(=O)=O)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCO', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CCOC(=O)CCN(Cc1ccccc1)SN(C)C(=O)O\\\\N=C(\\\\C)SC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/C(CC)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[NH3+][C@@H](CCC(=O)N[C@@H](CS/C(=N\\\\O)/CC1=CC=CC=C1)C(=O)NCC(=O)[O-])C(=O)[O-]', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCCC2=CC=CC=C2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('N[C@@H](CS\\\\C(Cc1ccc(O)cc1)=N/O)C(O)=O', 'Contains "
               "N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)C(=NOS(O)(=O)=O)CC=2C3=C(NC2)C=CC=C3OC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CS(=O)(=O)CCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CSCCCCCCCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CSCCCCCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)C(=NOS(O)(=O)=O)CC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CC(CO)C(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CC(O)(CC)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('COc1ccc(CC(S[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)=NOS(O)(=O)=O)cc1', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCCC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCSC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC2=CC(=CC=C2)OC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('COc1cccc2[nH]cc(CC(S[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)=NOS(O)(=O)=O)c12', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)C(=NOS(O)(=O)=O)CC(C)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[NH3+][C@@H](CS/C(=N/O)/CC=1C=2C=CC=CC2NC1)C(=O)NCC(=O)[O-]', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2cn(c3ccccc23)S(O)(=O)=O)=NOS([O-])(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CS(=O)(=O)CCCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCC=2C=CC=CC2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS([O-])(=O)=O)C[C@@H](C=C)O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2ccc(O)cc2)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CSC(=NO)C(=O)N(C)C', 'Contains N-hydroxyimidothioate "
               "group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/C(C)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCSC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCC(CC)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/*', "
               "'Contains N-hydroxyimidothioate group'), "
               "('COc1cccc2[nH]cc(CC(S[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)=NOS([O-])(=O)=O)c12', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CC=2C3=C(N(OC)C2)C=CC=C3OC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CC/C=C/S(=O)C', "
               "'Contains N-hydroxyimidothioate group'), ('*SC(*)=NO', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(CC(O)CC=C)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS(O)(=O)=O)CC(C=C)O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](S\\\\C([*])=N/O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCCCC2=CC=CC=C2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('N[C@@H](CS\\\\C([*])=N/O)C(O)=O', 'Contains "
               "N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/C(CC)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS([O-])(=O)=O)CC(C=C)O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CC(CC)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CSCCC\\\\C(SC[C@H](N)C(O)=O)=N\\\\O', 'Contains "
               "N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCSC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC(C)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCCS(C)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('COn1cc(CC(S[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)=NOS([O-])(=O)=O)c2ccccc12', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCS(C)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC(CC)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](S\\\\C(CCCOC(=O)C2=CC=CC=C2)=N\\\\OS([O-])(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[NH3+][C@@H](CS/C(=N\\\\O)/CC=1C=CC=CC1)C(=O)NCC(=O)[O-]', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/C[C@H](C=C)O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2c[nH]c3cccc(O)c23)=NOS([O-])(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CSCCCCCCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS(O)(=O)=O)C[C@H](C=C)O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CC=2C=CC=CC2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCCCCS(C)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CS(=O)CCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/*C=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[NH3+][C@@H](CCC(=O)N[C@@H](CS\\\\C(=N\\\\O)\\\\CC1=CNC2=C1C=CC=C2)C(=O)NCC(=O)[O-])C(=O)[O-]', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/C(C)C', "
               "'Contains N-hydroxyimidothioate group'), ('CSC(C)=NO', "
               "'Contains N-hydroxyimidothioate group'), "
               "('C1=CC=C2C(=C1)C(C/C(=N\\\\OS(O)(=O)=O)/S[C@]3([C@]([C@@]([C@]([C@](CO)([H])O3)([H])O)([H])O)([H])O)[H])=CN2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('COn1cc(CC(S[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)=NOS(O)(=O)=O)c2ccccc12', "
               "'Contains N-hydroxyimidothioate group'), "
               "('N[C@@H](CS\\\\C(Cc1c[nH]c2ccccc12)=N/O)C(O)=O', 'Contains "
               "N-hydroxyimidothioate group'), "
               "('OC[C@H]1O[C@@H](SC(Cc2c[nH]c3ccccc23)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)C(=NOS(O)(=O)=O)CCCCCCS(C)=O', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('CSC(C)=NOC(=O)NSNC(=O)ON=C(C)SC', 'Contains "
               "N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCCCCC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCC=2C=CC=CC2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCC=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)/C(=N\\\\OS(O)(=O)=O)/C[C@H](O)C=C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CC(O)C2=CC=CC=C2', "
               "'Contains N-hydroxyimidothioate group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CC2=CC(=CC=C2)OC', "
               "'Contains N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)COC(=O)/C=C/C2=CC(OC)=C(O)C(OC)=C2)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CC/C=C/S(=O)C', "
               "'Contains N-hydroxyimidothioate group'), "
               "('N[C@@H](CS\\\\C(Cc1ccccc1)=N/O)C(O)=O', 'Contains "
               "N-hydroxyimidothioate group'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCC2=CC=CC=C2', "
               "'Contains N-hydroxyimidothioate group')]\n"
               'False negatives: []',
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 25,
    'num_true_negatives': 183890,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.07407407407407407,
    'recall': 1.0,
    'f1': 0.13793103448275862,
    'accuracy': 0.9998640691181349}


"""
Classifies: CHEBI:134251 guaiacols
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_guaiacols(smiles: str):
    """
    Determines if a molecule contains a guaiacol moiety (phenol with methoxy substituent in ortho position).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains guaiacol, False otherwise
        str: Reason for classification
    """
    # Try to create mol object with sanitization
    try:
        mol = Chem.MolFromSmiles(smiles, sanitize=True)
    except:
        return None, "Invalid SMILES string or molecule could not be sanitized"
        
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for guaiacol:
    # Phenol with methoxy in ortho position
    # c: aromatic carbon
    # O: oxygen
    # [CH3]: methyl group
    guaiacol_pattern = '[OH]c1c(OC)cccc1'
    
    try:
        substructure = Chem.MolFromSmarts(guaiacol_pattern)
        if mol.HasSubstructMatch(substructure):
            return True, "Contains guaiacol moiety (phenol with ortho-methoxy group)"
        return False, "Does not contain guaiacol moiety"
    except:
        return None, "Error in substructure matching"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134251',
                          'name': 'guaiacols',
                          'definition': 'Any phenol carrying an additional '
                                        'methoxy substituent at the '
                                        'ortho-position.',
                          'parents': ['CHEBI:25235', 'CHEBI:33853']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: Can't kekulize mol.  Unkekulized atoms: 0 1 2 "
               '3 4 5 6 7 9 10 11 13 14 15 16 18 19 20 21 22 23 24 25 26 27 28 '
               '29 30 31 32 33 34 35 36 37 38 39 41 42 44 45 46 47 48 50 51 53 '
               '54 55 56 57 58 59 60 61 69 70 71 72 73 74 75 76 78 80 81 82 83 '
               '88 89',
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 9,
    'num_false_positives': 100,
    'num_true_negatives': 4478,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.08256880733944955,
    'recall': 1.0,
    'f1': 0.15254237288135594,
    'accuracy': 0.9781992587747983}


"""
Classifies: CHEBI:25513 neutral glycosphingolipid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_neutral_glycosphingolipid(smiles: str):
    """
    Determines if a molecule is a neutral glycosphingolipid.
    A neutral glycosphingolipid contains unsubstituted glycosyl moieties attached to a ceramide core.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a neutral glycosphingolipid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for ceramide core
    ceramide_substructure = "[CH2]OC1O[CH]([CH]O)[CH]([CH]O)[CH]([CH]1O)O" # Basic ceramide core pattern
    ceramide_pattern = Chem.MolFromSmarts(ceramide_substructure)
    if not mol.HasSubstructMatch(ceramide_pattern):
        return False, "No ceramide core found"

    # Check for glycosyl groups
    glycosyl_substructure = "OC1[CH]([CH]O)[CH]([CH]O)[CH]O[CH]1" # Basic glycosyl pattern
    glycosyl_pattern = Chem.MolFromSmarts(glycosyl_substructure)
    if not mol.HasSubstructMatch(glycosyl_pattern):
        return False, "No glycosyl groups found"

    # Check for absence of charged/acidic groups that would make it non-neutral
    charged_groups = [
        "[O-]", # Carboxylate
        "[N+]", # Quaternary amine 
        "[S](=O)(=O)[O-]", # Sulfate
        "[P](=O)([O-])[O-]" # Phosphate
    ]
    
    for group in charged_groups:
        pattern = Chem.MolFromSmarts(group)
        if mol.HasSubstructMatch(pattern):
            return False, f"Contains charged group: {group}"

    # Count number of glycosyl units
    glycosyl_matches = len(mol.GetSubstructMatches(glycosyl_pattern))
    
    # Check for presence of ceramide and glycosyl parts
    if glycosyl_matches >= 1:
        return True, f"Neutral glycosphingolipid with {glycosyl_matches} glycosyl unit(s)"
    else:
        return False, "Missing required structural components"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25513',
                          'name': 'neutral glycosphingolipid',
                          'definition': 'Any glycosphingolipid containing '
                                        'unsubstituted glycosyl moieties.',
                          'parents': ['CHEBI:24402']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183235,
    'num_false_negatives': 75,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999590857018166}


"""
Classifies: CHEBI:33646 alkadiene
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_alkadiene(smiles: str):
    """
    Determines if a molecule is an alkadiene (acyclic branched or unbranched hydrocarbons having two carbon-carbon double bonds).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an alkadiene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule is acyclic
    if mol.HasSubstructMatch(Chem.MolFromSmarts('[r]')):
        return False, "Molecule is cyclic"

    # Check if the molecule consists of carbon and hydrogen atoms only
    atoms = [atom.GetSymbol() for atom in mol.GetAtoms()]
    if any(atom not in ['C', 'H'] for atom in atoms):
        return False, "Molecule contains atoms other than carbon and hydrogen"

    # Count the number of double bonds
    double_bonds = sum(1 for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.DOUBLE)
    if double_bonds != 2:
        return False, f"Molecule has {double_bonds} double bonds, expected 2"

    # Check if the double bonds are separated by at least one carbon atom
    double_bond_atoms = [bond.GetBeginAtomIdx() for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.DOUBLE]
    double_bond_atoms.extend([bond.GetEndAtomIdx() for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.DOUBLE])
    distances = [Chem.GetDistanceMatrix(mol)[i, j] for i, j in zip(double_bond_atoms[:-1], double_bond_atoms[1:])]
    if any(distance < 2 for distance in distances):
        return False, "Double bonds are not separated by at least one carbon atom"

    return True, "Molecule is an alkadiene"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33646',
                          'name': 'alkadiene',
                          'definition': 'Acyclic branched or unbranched '
                                        'hydrocarbons having two carbon-carbon '
                                        'double bonds.',
                          'parents': ['CHEBI:33645']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.17391304347826084 is too low.\n'
               "True positives: [('C(CCCC=C)CCCC=C', 'Molecule is an "
               "alkadiene'), ('C(CC=CC)C=CC', 'Molecule is an alkadiene')]\n"
               "False positives: [('C(CC=C(C)C)/C(/C)=C\\\\C', 'Molecule is an "
               "alkadiene'), ('C(C/C=C/C(/C)=C/C)(C)C', 'Molecule is an "
               "alkadiene'), ('C(/C=C/C=C(C)C)(C)C', 'Molecule is an "
               "alkadiene'), ('C(CCCC(CCCC(C=C)=C)C)(CCCC(C)C)C', 'Molecule is "
               "an alkadiene'), ('C=C=C', 'Molecule is an alkadiene'), "
               "('C(CC(C)=C)CC=C', 'Molecule is an alkadiene'), "
               "('C(C#C)#CC=C=C', 'Molecule is an alkadiene'), "
               "('C(\\\\CC)(=C/CC=C(C)C)/C', 'Molecule is an alkadiene'), "
               "('C(CC=C(C)C)/C(/C)=C/C', 'Molecule is an alkadiene'), "
               "('[C@@H](CCC[C@@H](CCCC(C)C)C)(CCC/C(=C/CC/C=C(/CCC[C@@H](CCC[C@@H](CCCC(C)C)C)C)\\\\C)/C)C', "
               "'Molecule is an alkadiene'), ('C(=C/C=C/C)(C)C', 'Molecule is "
               "an alkadiene'), ('C(CC)C=C(C(C)=C)C', 'Molecule is an "
               "alkadiene'), ('C(C(C/C=C/C)=C)(C)C', 'Molecule is an "
               "alkadiene'), ('C(\\\\CC=C)(=C/CC)/C', 'Molecule is an "
               "alkadiene'), ('C=C[C@@](C)(CCC=C(C)C)[H]', 'Molecule is an "
               "alkadiene'), ('C(CC)/C(/C)=C/C=C', 'Molecule is an "
               "alkadiene'), ('C(C=C(C)C)(C)C=C', 'Molecule is an alkadiene'), "
               "('CCCC/C=C/C=C', 'Molecule is an alkadiene'), "
               "('CCC(=C)\\\\C=C/CC(C)C', 'Molecule is an alkadiene')]\n"
               'False negatives: []',
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 9,
    'num_true_negatives': 183898,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.18181818181818182,
    'recall': 1.0,
    'f1': 0.3076923076923077,
    'accuracy': 0.9999510627538619}


"""
Classifies: CHEBI:142621 medium-chain fatty aldehyde
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_medium_chain_fatty_aldehyde(smiles: str):
    """
    Determines if a molecule is a medium-chain fatty aldehyde (C6-C12).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a medium-chain fatty aldehyde, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of aldehyde group (C=O)
    aldehyde_pattern = Chem.MolFromSmarts('[CH1](=O)')
    if not mol.HasSubstructMatch(aldehyde_pattern):
        return False, "No aldehyde group found"

    # Count carbons in the main chain
    # First, find the aldehyde carbon
    matches = mol.GetSubstructMatches(aldehyde_pattern)
    if not matches:
        return False, "No aldehyde group found"
    
    aldehyde_carbon = matches[0][0]
    
    # Traverse the molecule from the aldehyde carbon to count the chain length
    visited = set()
    current_atom = mol.GetAtomWithIdx(aldehyde_carbon)
    chain_length = 1  # Start with 1 for the aldehyde carbon
    
    while True:
        # Get non-oxygen neighbors (to follow the carbon chain)
        carbon_neighbors = [neighbor for neighbor in current_atom.GetNeighbors() 
                          if neighbor.GetSymbol() == 'C' and 
                          neighbor.GetIdx() not in visited]
        
        if not carbon_neighbors:
            break
            
        # Follow the longest carbon chain
        current_atom = carbon_neighbors[0]
        visited.add(current_atom.GetIdx())
        chain_length += 1

    if chain_length < 6:
        return False, f"Chain too short (C{chain_length})"
    elif chain_length > 12:
        return False, f"Chain too long (C{chain_length})"
    else:
        # Count double bonds pattern [C]=[C]
        double_bond_pattern = Chem.MolFromSmarts('[C]=[C]')
        double_bonds = len(mol.GetSubstructMatches(double_bond_pattern))
        
        if double_bonds > 0:  # Has additional double bonds besides aldehyde
            return True, f"Unsaturated medium-chain fatty aldehyde (C{chain_length})"
        else:
            return True, f"Saturated medium-chain fatty aldehyde (C{chain_length})"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:142621',
                          'name': 'medium-chain fatty aldehyde',
                          'definition': 'Any fatty aldehyde with a chain '
                                        'length between C6 and C12.',
                          'parents': ['CHEBI:35746']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.rdMolDescriptors' has no "
               "attribute 'CalcNumAliphaticDoubleBonds'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 12687,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9921807803581203}


"""
Classifies: CHEBI:22478 amino alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.MolStandardize import rdMolStandardize

def is_amino_alcohol(smiles: str):
    """
    Determines if a molecule is an amino alcohol (contains both amino and hydroxyl groups).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an amino alcohol, False otherwise
        str: Reason for classification
    """
    # Convert SMILES to molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Standardize the molecule (uncharge, neutralize, etc)
    uncharger = rdMolStandardize.Uncharger()
    mol = uncharger.uncharge(mol)

    # Find alcohol (hydroxyl) groups
    alcohol_pattern = Chem.MolFromSmarts("[#6][OX2H]")
    alcohols = mol.GetSubstructMatches(alcohol_pattern)
    
    # Find primary and secondary amino groups
    amino_patterns = [
        Chem.MolFromSmarts("[NX3;H2][CX4]"), # Primary amine
        Chem.MolFromSmarts("[NX3;H1]([CX4])[CX4]"), # Secondary amine
        Chem.MolFromSmarts("[NX3]([CX4])([CX4])[CX4]"), # Tertiary amine
        Chem.MolFromSmarts("[NX3]"), # Any amine
    ]
    
    amino_groups = []
    for pattern in amino_patterns:
        matches = mol.GetSubstructMatches(pattern)
        amino_groups.extend(matches)
    
    # Remove duplicates
    amino_groups = list(set(amino_groups))
    
    if not alcohols:
        return False, "No alcohol groups found"
        
    if not amino_groups:
        return False, "No amino groups found"
        
    # Count the functional groups
    n_alcohols = len(alcohols)
    n_amines = len(amino_groups)
    
    return True, f"Found {n_alcohols} alcohol group(s) and {n_amines} amino group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22478',
                          'name': 'amino alcohol',
                          'definition': 'An alcohol containing an amino '
                                        'functional group in addition to the '
                                        'alcohol-defining hydroxy group.',
                          'parents': ['CHEBI:30879', 'CHEBI:50047']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 33,
    'num_false_positives': 100,
    'num_true_negatives': 137,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.24812030075187969,
    'recall': 1.0,
    'f1': 0.3975903614457831,
    'accuracy': 0.6296296296296297}


"""
Classifies: CHEBI:33259 elemental molecular entity
"""
from rdkit import Chem

def is_elemental_molecular_entity(smiles: str):
    """
    Determines if a molecule is an elemental molecular entity (all atoms have the same atomic number).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an elemental molecular entity, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get the list of atomic numbers present in the molecule
    atomic_nums = set(atom.GetAtomicNum() for atom in mol.GetAtoms())

    # Check if there is only one unique atomic number
    if len(atomic_nums) == 1:
        return True, f"All atoms have the same atomic number: {list(atomic_nums)[0]}"
    else:
        return False, "Molecule contains atoms with different atomic numbers"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33259',
                          'name': 'elemental molecular entity',
                          'definition': 'A molecular entity all atoms of which '
                                        'have the same atomic number.',
                          'parents': ['CHEBI:23367']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 24,
    'num_false_positives': 100,
    'num_true_negatives': 10826,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.1935483870967742,
    'recall': 0.8571428571428571,
    'f1': 0.3157894736842105,
    'accuracy': 0.9905057513237173}


"""
Classifies: CHEBI:140331 4'-hydroxyflavanones
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_4__hydroxyflavanones(smiles: str):
    """
    Determines if a molecule is a 4'-hydroxyflavanone (has a hydroxy substituent at position 4').
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a 4'-hydroxyflavanone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define SMARTS pattern for flavanone core with 4'-OH
    # The pattern specifically matches:
    # - The chroman-4-one core (O-C-C-C(=O))
    # - The B-ring phenyl with OH at para position
    # - Ensures proper connectivity
    flavanone_pattern = Chem.MolFromSmarts('[OH]-c1ccc(cc1)[CH2,CH]2CC(=O)c3c(O2)cc([#6,#8,H])cc3')
    
    if flavanone_pattern is None:
        return None, "Invalid SMARTS pattern"

    # Check if molecule contains the pattern
    if mol.HasSubstructMatch(flavanone_pattern):
        # Additional validation to ensure proper stereochemistry and structure
        matches = mol.GetSubstructMatches(flavanone_pattern)
        
        # Check if any match satisfies all conditions
        for match in matches:
            # The pattern ensures:
            # - Flavanone core with proper connectivity
            # - 4'-hydroxy group
            # - Proper aromatic rings
            return True, "Contains 4'-hydroxyflavanone structure"
            
    return False, "Does not contain 4'-hydroxyflavanone structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140331',
                          'name': "4'-hydroxyflavanones",
                          'definition': 'Any hydroxyflavanone having a hydroxy '
                                        "substituent located at position 4'.",
                          'parents': ['CHEBI:24697']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.HasSubstructMatch(Mol, NoneType)\n'
               'did not match C++ signature:\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, RDKit::SubstructMatchParameters params=True)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'RDKit::SubstructMatchParameters params)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, bool recursionPossible=True, bool useChirality=False, '
               'bool useQueryQueryMatches=False)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'bool recursionPossible=True, bool useChirality=False, bool '
               'useQueryQueryMatches=False)',
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 9,
    'num_false_positives': 100,
    'num_true_negatives': 104621,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.08256880733944955,
    'recall': 1.0,
    'f1': 0.15254237288135594,
    'accuracy': 0.9990451637544161}


"""
Classifies: CHEBI:140402 tert-butyl ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_tert_butyl_ester(smiles: str):
    """
    Determines if a molecule contains a tert-butyl ester group.
    A tert-butyl ester is a carboxylic ester with a tert-butyl group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains a tert-butyl ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for tert-butyl ester: R-C(=O)-O-C(C)(C)C
    # Exclude carbamates and carbonates
    tert_butyl_ester_pattern = Chem.MolFromSmarts('[C,c;!$(C(=O)O);!$(C(=O)N)][CX3](=[OX1])[OX2][CX4;!$(C(O)(O))]([CH3])([CH3])[CH3]')
    
    if tert_butyl_ester_pattern is None:
        return None, "Invalid SMARTS pattern"
        
    matches = mol.GetSubstructMatches(tert_butyl_ester_pattern)
    
    # Check for carbonate groups
    carbonate_pattern = Chem.MolFromSmarts('[OX2][CX3](=[OX1])[OX2]')
    carbonate_matches = mol.GetSubstructMatches(carbonate_pattern) if carbonate_pattern else []
    
    # Filter out matches that are part of a carbonate
    valid_matches = []
    for match in matches:
        is_carbonate = False
        for carbonate in carbonate_matches:
            if match[1] in carbonate:  # Check if carbonyl carbon is part of carbonate
                is_carbonate = True
                break
        if not is_carbonate:
            valid_matches.append(match)
            
    if valid_matches:
        num_matches = len(valid_matches)
        if num_matches == 1:
            return True, "Contains 1 tert-butyl ester group"
        else:
            return True, f"Contains {num_matches} tert-butyl ester groups"
    
    return False, "No tert-butyl ester group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140402',
                          'name': 'tert-butyl ester',
                          'definition': 'A carboxylic ester resulting from the '
                                        'formal condensation of a carboxylic '
                                        'acid with tert-butanol.',
                          'parents': ['CHEBI:33308']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.2857142857142857 is too low.\n'
               "True positives: [('C(OC(C)(C)C)(C=1C=CC=CC1)=O', 'Contains 1 "
               "tert-butyl ester group'), "
               "('CC(C)C[C@@H](C(=O)OC(C)(C)C)NC(=O)C1=C(NC=N1)C(=O)NCC2=CC=CC=C2', "
               "'Contains 1 tert-butyl ester group'), "
               "('CCOC(=O)C1=CC=C(C=C1)NC(=O)C2=C(NC=N2)C(=O)N[C@@H](CCCCNC(=O)OC(C)(C)C)C(=O)OC(C)(C)C', "
               "'Contains 1 tert-butyl ester group')]\n"
               "False positives: [('C(*)(=O)OC(C)(C)C', 'Contains 1 tert-butyl "
               "ester group'), ('O(C(C)(C)C)C(=O)C1=CC(N)=CC=C1', 'Contains 1 "
               "tert-butyl ester group'), ('O(C(C)(C)C)C(=O)C1=CC=C(C=C1)C', "
               "'Contains 1 tert-butyl ester group'), "
               "('CC(C)(C)OC(=O)OC(=O)OC(C)(C)C', 'Contains 2 tert-butyl ester "
               "groups'), "
               "('O(C(=O)[C@H]1N(CCC1)C(=O)[C@@H](N[C@@H](CCC2=CC=CC=C2)C(O)=O)C)C(C)(C)C', "
               "'Contains 1 tert-butyl ester group'), "
               "('O(C(C)(C)C)C(=O)C(NC(=O)CCCCC)C(C)C', 'Contains 1 tert-butyl "
               "ester group'), ('C(OC(C)(C)C)(*)=O', 'Contains 1 tert-butyl "
               "ester group'), ('O(C(C)(C)C)C(=O)[C@@H](N)CCC(=O)N', 'Contains "
               "1 tert-butyl ester group'), "
               "('O(C(C)(C)C)C(=O)[C@H](N)CCC(O)=O', 'Contains 1 tert-butyl "
               "ester group'), "
               "('ClC1=CC=2C3(CCNC3=O)C(=O)N(C2C=C1)CC(OC(C)(C)C)=O', "
               "'Contains 1 tert-butyl ester group'), "
               "('O(C(C)(C)C)C(=O)C1=CC=C(CN2CCN(CC2)C=3C4=C(NC=C4)C=CC3)C=C1', "
               "'Contains 1 tert-butyl ester group'), "
               "('ClC=1SC(C(OC(C)(C)C)=O)=CN1', 'Contains 1 tert-butyl ester "
               "group'), "
               "('O(C(=O)[C@@H]([C@H](N([C@@H](C1=CC=CC=C1)C)CC2=CC=CC=C2)C=3C(N(CC=C)CC=C)=CC=CC3)CC#N)C(C)(C)C', "
               "'Contains 1 tert-butyl ester group'), "
               "('P(C1=CC=CC=C1)(C2=CC=CC=C2)(C3=CC=CC=C3)=CC(OC(C)(C)C)=O', "
               "'Contains 1 tert-butyl ester group'), "
               "('S(C(CC=C)(C1=CC=C(OC)C=C1)C(OC(C)(C)C)=O)CC=C', 'Contains 1 "
               "tert-butyl ester group')]\n"
               'False negatives: []',
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 12,
    'num_true_negatives': 183885,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.2,
    'recall': 1.0,
    'f1': 0.33333333333333337,
    'accuracy': 0.9999347471451876}


"""
Classifies: CHEBI:25481 naphthoquinone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdDecomposition

def is_naphthoquinone(smiles: str):
    """
    Determines if a molecule is a naphthoquinone.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a naphthoquinone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for naphthalene core
    naphthalene_pattern = Chem.MolFromSmarts('c1ccc2ccccc2c1')
    if not mol.HasSubstructMatch(naphthalene_pattern):
        return False, "No naphthalene core found"
    
    # Check for two ketone groups
    ketone_pattern = Chem.MolFromSmarts('C(=O)')
    ketone_matches = mol.GetSubstructMatches(ketone_pattern)
    if len(ketone_matches) < 2:
        return False, "Less than 2 ketone groups found"

    # Check specifically for 1,4-naphthoquinone pattern
    naphthoquinone_pattern = Chem.MolFromSmarts('O=C1C=CC(=O)c2ccccc21')
    naphthoquinone_pattern2 = Chem.MolFromSmarts('O=C1C(=O)c2ccccc2C=C1')
    
    if not (mol.HasSubstructMatch(naphthoquinone_pattern) or mol.HasSubstructMatch(naphthoquinone_pattern2)):
        # Check for other naphthoquinone patterns with substituents
        nq_pattern3 = Chem.MolFromSmarts('O=C1C(*)C(*)C(=O)c2ccccc21')
        nq_pattern4 = Chem.MolFromSmarts('O=C1C(*)=C(*)C(=O)c2ccccc21')
        if not (mol.HasSubstructMatch(nq_pattern3) or mol.HasSubstructMatch(nq_pattern4)):
            return False, "No naphthoquinone core structure found"

    # Get number of substituents
    substituents = []
    core_atoms = set()
    for match in mol.GetSubstructMatches(naphthalene_pattern):
        core_atoms.update(match)
    
    for atom in mol.GetAtoms():
        if atom.GetIdx() not in core_atoms and atom.GetSymbol() != 'H':
            substituents.append(atom.GetSymbol())

    return True, f"Naphthoquinone with {len(set(substituents))} types of substituents"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25481',
                          'name': 'naphthoquinone',
                          'definition': 'A polycyclic aromatic ketone '
                                        'metabolite of naphthalene.',
                          'parents': ['CHEBI:36141']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'rdDecomposition' from 'rdkit.Chem' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:138088 long chain primary alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_long_chain_primary_alcohol(smiles: str):
    """
    Determines if a molecule is a long chain primary alcohol (chain length 13-22 carbons).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a long chain primary alcohol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of primary alcohol group (CH2OH)
    primary_alcohol_pattern = Chem.MolFromSmarts("[CH2][OH]")
    if not mol.HasSubstructMatch(primary_alcohol_pattern):
        return False, "Not a primary alcohol"

    # Get the carbon atom connected to OH
    matches = mol.GetSubstructMatches(primary_alcohol_pattern)
    ch2_atom = mol.GetAtomWithIdx(matches[0][0])

    # Count total carbons and check for linearity
    total_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    ring_info = mol.GetRingInfo()
    rings = ring_info.AtomRings()
    
    # If there are rings, check their size and nature
    if rings:
        ring_carbons = set()
        for ring in rings:
            ring_carbons.update(idx for idx in ring if mol.GetAtomWithIdx(idx).GetSymbol() == 'C')
            
        # If CH2OH is part of a ring or connected to a ring atom, reject
        for neighbor in ch2_atom.GetNeighbors():
            if neighbor.GetIdx() in ring_carbons:
                return False, "Primary alcohol is directly connected to ring system"
                
        # If most carbons are in rings, reject
        if len(ring_carbons) > total_carbons / 2:
            return False, "Structure is predominantly cyclic"

    # For wildcard atoms, assume it's a valid chain
    if '*' in smiles:
        return True, "Valid long chain primary alcohol with wildcard atoms"

    # Calculate the longest chain from CH2OH
    def get_longest_chain(atom, visited=None):
        if visited is None:
            visited = set()
        
        visited.add(atom.GetIdx())
        max_length = 0
        
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in visited and neighbor.GetSymbol() == 'C':
                length = get_longest_chain(neighbor, visited.copy())
                max_length = max(max_length, length)
                
        return max_length + 1

    chain_length = get_longest_chain(ch2_atom)

    if chain_length < 13:
        return False, f"Chain too short ({chain_length} carbons)"
    elif chain_length > 22:
        return False, f"Chain too long ({chain_length} carbons)"

    # Check for excessive branching
    branching_count = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetDegree() > 2:
            branching_count += 1
            if branching_count > 2:  # Allow some branching but not too much
                return False, "Structure is too branched"

    return True, f"Long chain primary alcohol with {chain_length} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138088',
                          'name': 'long chain primary alcohol',
                          'definition': 'A primary alcohol with a chain length '
                                        'ranging from 13-22 carbons which is '
                                        'usually but not always a fatty '
                                        'alcohol.',
                          'parents': ['CHEBI:15734']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.056074766355140186 is too low.\n'
               "True positives: [('CCC(C)CCCCCCCCCCCCCCCCCCCO', 'Long chain "
               "primary alcohol with 22 carbons'), ('CC(C)CCCCCCCCCCCCCCO', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('CCCCCCCCCCCCCCCCCO', 'Long chain primary alcohol with 17 "
               "carbons')]\n"
               'False positives: '
               "[('[H][C@@]12CC[C@](C)(CC1=CC[C@@]1([H])[C@@](C)(CO)CCC[C@]21C)C=C', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('COc1cc(C[C@@H]2[C@@H](Cc3cc(OC)c(O)c(c3)C(CO)C(O)c3ccc(O)c(OC)c3)COC2=O)ccc1O', "
               "'Long chain primary alcohol with 17 carbons'), "
               "('O=C1C2=C(OC3=C1C(C(=O)OC)=CC=C3O)C(OC)=C(C(C(O)CO)CO)C=C2OC', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('CC1(C)O[C@@H]2C[C@H]3[C@@H]4C[C@H](F)C5=CC(=O)C=C[C@]5(C)[C@H]4[C@@H](O)C[C@]3(C)[C@@]2(O1)C(=O)CO', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('[C@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)(NC(=O)*COC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)CO', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('O[C@@H]1C([C@H]2[C@](C3=C([C@]4([C@]([C@@H]([C@@H]([C@H](O)CC=C(C)C)CO)CC4)(C)CC3)C)CC2)(C)CC1)(C)C', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O=C1C2=C([C@]3(CCC(C([C@@H]3C1)(C)C)=O)C)CC[C@]4([C@]2(CC[C@@H]4[C@H](CO)CC/C=C(/CO)\\\\C)C)C', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('CCNC(=O)C1=NC(=C2[C@@H](N(CC2=C1)[S@@](=O)C(C)(C)C)CCO)C3=CC(=CC=C3)C4=CCCC4', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('[H][C@@]12CCC3CC(=O)CC[C@]3(C)[C@@]1([H])C(=O)C[C@@]1(C)[C@@]2([H])CC[C@]1(O)C(=O)CO', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O=C1C=2C(=CC[C@H]3[C@]4(O[C@@H](/C=C(/CO)\\\\C)C[C@@H]4C)CC[C@@]3(CC2[C@@H](C1)C)C)CO', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('C1[C@H]2[C@@H]([C@H]([C@H](N2)C3=CC=C(C(=O)N31)C4=CC=C(C=C4)F)C(=O)NCCC5=CC=NC=C5)CO', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('O=C1[C@]2([C@]([C@]3([C@@]([C@H](CC3)C(=O)CO)(C1)CO)[H])(CC[C@]4([C@@]2(CC[C@@H](O)C4)C)[H])[H])[H]', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('OCCCCC\\\\C=C/C[C@@H](O)C(O)\\\\C=C\\\\C(O)C\\\\C=C/CCCC(O)=O', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('COCC(=O)N1C[C@@H]2[C@H]([C@@H](N2C(=O)C1)CO)C3=CC=C(C=C3)C4=CC=CC=C4F', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('CC(C)NC[C@@H]1[C@@H]([C@@H](N1C(=O)NC(C)C)CO)C2=CC=C(C=C2)C3=CCCCC3', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('O[C@]([C@@H](O)[C@@H](O)C[C@@]1([C@H]2C3=C4C(=C(C)CC4=C(C)C=C3)C[C@@]2(C)C[C@@H]1O)C)(CO)C', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('C1CN([C@@H]2[C@H]1[C@@H](NC3=C2C=C(C=C3)C#CC4=CC=CC=C4)CO)C(=O)C5=CC(=CC=C5)F', "
               "'Long chain primary alcohol with 17 carbons'), "
               "('C1=2C(OC(=CC1=O)C3=CC=C(C(=C3)OC)O)=CC(=C(C2O)[C@H]4[C@@H]([C@H]([C@@H]([C@H](O4)CO)O)O)O)OC', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O[C@H](C(=C)CC/C=C(/CO)\\\\C)CC/C(=C/CC/C(=C/CO)/C)/C', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('C1CCC(C1)NC(=O)[C@@H]2[C@H]([C@@H]3CN4C(=CC=C(C4=O)C5=CC=CC=C5)[C@H]2N3CCC6=CC=CC=C6)CO', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('CCCCCCCCCCCCC(O)CO', 'Long chain primary alcohol with 14 "
               "carbons'), "
               "('C1CCC(CC1)C#CC2=CC=C(C=C2)[C@@H]3[C@@H]4CN(CC(=O)N4[C@H]3CO)C(=O)CC5=CN=CC=C5', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('Cl[C@H]([C@H](O)[C@H](O)[C@H](O)/C=C/C1=C(C(OC)=CC=C1)CO)C', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('O=C1C(OC)=C(C(=O)C2=C1C=C(O)C3=C2O[C@@H](C)[C@]3(C(O)C/C=C(/CO)\\\\C)C)C', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('C(C\\\\C=C(\\\\CC\\\\C=C(\\\\CC\\\\C=C(\\\\CCC=C(C)C)/C)/C)/C)/C(=C\\\\CO)/C', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O1[C@]([C@H](O)[C@@H](O)[C@H](O)[C@H]1CO)(C2=C(O)C([C@@]3(O[C@@H]([C@@H](O)[C@H](O)[C@H]3O)CO)[H])=C(O)C(=C2O)C(=O)CCC4=CC=C(O)C=C4)[H]', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O=C/1O[C@](CO)(C)C\\\\C1=C\\\\CCCCCCC/C=C\\\\CCCCCC', 'Long "
               "chain primary alcohol with 20 carbons'), "
               "('[H][C@]12O[C@]11CC[C@]3([H])C(C)(C)CCC[C@@]3(C)[C@]1([H])[C@@]1([H])O[C@]11OC(=O)C(CO)=C21', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('O=C(O[C@@H]1[C@]23O[C@@](OC)(C=C([C@H]2CC3(C)C)CO)C=C(C1)C)C', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('C1CCN(CC1)C(=O)[C@H]2[C@@H]([C@H]3CN4C(=CC=C(C4=O)C5=CCCCC5)[C@@H]2N3CC6=NC=CS6)CO', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('OC1=C([C@H](CCN(C(C)C)C(C)C)C2=CC=CC=C2)C=C(C=C1)CO', 'Long "
               "chain primary alcohol with 13 carbons'), "
               "('O1C(C(O)C(O)C(O)C1CO)C2=C(O)C(=C(O)C=C2O)C(=O)C', 'Long "
               "chain primary alcohol with 13 carbons'), "
               "('C(\\\\C=C/C=C/C=C/[C@H]([C@H](CCCCCO)O)O)=C/[C@H](CCCC([O-])=O)O', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O=C1C=C2[C@@]3(CC[C@]([C@H](C2)C3)(O)CO)[C@@]4([C@@H]1[C@@]([C@H](O)CC4)(CO)C)C', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('O=C(O[C@H]1[C@H](O[C@@H]2O[C@@H]([C@H](O)[C@@H]([C@H]2O)O)COC(=O)/C=C/C=C\\\\CCCCC)[C@H](OC([C@@H]1O)C3=C(O)C=C(O)C=C3CO)CO)/C(=C/C=C/CC(O)C(CC)C)/C', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('[H][C@@]12C=C(CO)C[C@]3(O)C(=O)C(C)=C[C@@]3([H])[C@@]1(O)[C@H](C)[C@@H](OC(=O)CC\\\\C=C/CCCCC)[C@]1(OC(C)=O)[C@@]2([H])C1(C)C', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('CC1=CC=C(C=C1)S(=O)(=O)N2CC[C@H]3[C@@H]2C4=C(C=CC(=C4)C5=CC(=CC=C5)C(=O)N(C)C)N[C@@H]3CO', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('C1COCCC1C(=O)N2C[C@@H]3[C@H]([C@H](N3C(=O)C2)CO)C4=CC=C(C=C4)C#CCC5=CC=CC=C5', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('O=C(O)/C(=C/CC[C@H]([C@@H]1[C@@]2([C@@]([C@H]3[C@@]([C@@H](C(=C(C)C)CC3)CCCO)(C)CC2)(C)CC1)C)C)/C', "
               "'Long chain primary alcohol with 22 carbons'), "
               "('CN(C)C(=O)[C@H]1[C@@H]([C@H]2CN3C(=CC=C(C3=O)C4=CCCC4)[C@@H]1N2CC5CCOCC5)CO', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('CCC(=O)N1[C@@H]2CN3C(=CC=C(C3=O)C4=CC=C(C=C4)F)[C@H]1[C@H]([C@@H]2CO)C(=O)OC', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('COC1=CC=CC=C1C2=CC=C(C=C2)[C@@H]3[C@H](N[C@@H]3C#N)CO', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('OC[C@H]1O[C@H]([C@H](O)[C@@H](O)[C@@H]1O)c1c(O)c([C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)c2occ(-c3ccc(O)cc3)c(=O)c2c1O', "
               "'Long chain primary alcohol with 19 carbons'), "
               "('O1[C@@H]([C@H]1/C=C/C(=O)[C@@]2(O)[C@@H](O)[C@@H](NC2=O)CO)CCCCCCCCC', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('O1[C@H](O[C@@H]2C3=C(C(C)C)C[C@@H]([C@@]3(C=C4[C@H](CO)CC[C@H]4[C@H]([C@H]2O)C)C)O)[C@H](O)[C@@H](O)[C@@H]([C@H]1CO)O', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('C[C@]12C[C@H](O)[C@@]3(F)[C@@H](CCC4=CC(=O)C=C[C@]34C)[C@@H]1CC[C@]2(O)C(=O)CO', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('COC1=CC=CC(=C1)C2=CC3=C(C=C2)N[C@H]([C@H]4[C@@H]3N(CC4)C(=O)CC5=CC=CC=C5)CO', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('CC1=CC=CC=C1S(=O)(=O)N2CC[C@@H]3[C@H]2C4=C(C=CC(=C4)C5=CC(=CC=C5)OC)N[C@@H]3CO', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('O=C1C(=C(C=CCCCCC)[C@@H](O)[C@H](C1)O)CO', 'Long chain "
               "primary alcohol with 14 carbons'), "
               "('COC1=CC=CC(=C1)NC(=O)N2C[C@@H]3[C@@H]([C@H](N3C(=O)C2)CO)C4=CC=C(C=C4)C#CCC5CCCC5', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('C1CCC(CC1)C#CC2=CC=C(C=C2)[C@@H]3[C@@H]4CN(CC(=O)N4[C@H]3CO)C(=O)C5CCOCC5', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('C\\\\C(CO)=C/CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\C([O-])=O', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('C1CCC(=CC1)C2=CC=C3[C@@H]4[C@@H]([C@H]([C@@H](N4)CN3C2=O)CO)C(=O)NCC5=CC=C(C=C5)F', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('CCCCCCCCCCCC(=O)[C@@H]([NH3+])CO', 'Long chain primary "
               "alcohol with 14 carbons'), "
               "('O=C1C(=C)[C@]2([C@H]3[C@H](CC(C3)(C)C)C[C@]42[C@@H]1O4)CO', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('O1[C@](C(O)[C@@H](O)[C@H](O)C1CO)(C=2C=3OC=C(C(=O)C3C=CC2O)C4=CC=C(OC)C=C4)[H]', "
               "'Long chain primary alcohol with 19 carbons'), "
               "('O=C1N2[C@@]3([C@@]4([C@]5(N(CC4)C\\\\C(\\\\[C@](C5)([C@]3([C@@H](C1)O)[H])[H])=C\\\\CO)[H])C=6C2=CC=CC6)[H]', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('OC[C@@]12[C@]([C@]3([C@](CC1)([C@@]4(C(CC3)=CC(=O)C=C4)C)[H])[H])(CC[C@@H]2C=C)[H]', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('CCCCCCCCCCCCCCCC(=O)C(N)CO', 'Long chain primary alcohol "
               "with 18 carbons'), "
               "('Cc1coc2C(=O)c3c(ccc4c3CCCC4(C)CO)C(=O)c12', 'Long chain "
               "primary alcohol with 16 carbons'), "
               "('C\\\\C(CCC1=C(C)CCCC1(C)C)=C/CC\\\\C(CO)=C/C[C@H]1OC(=O)C=C1CO', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('O=C1C(O)=C(C(=O)C=C1CO)C[C@H]2C(=C)CC[C@@H]3[C@@]2(CCC[C@@]3(CO)C)C', "
               "'Long chain primary alcohol with 17 carbons'), "
               "('CCCCCCCCCCCCC\\\\C=C\\\\[C@H](O)[C@H](N)CO', 'Long chain "
               "primary alcohol with 18 carbons'), "
               "('O=C1C2=C(O)C(=C(O)C=C2C(=O)C=3C1=C(O)C=C(O)C3)[C@H](C(=O)O)CCO', "
               "'Long chain primary alcohol with 17 carbons'), "
               "('C(\\\\CC(/C=C/C=C\\\\C/C=C\\\\CCCCCO)OO)=C\\\\CCCC(OC)=O', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('CC(C)(C)[S@@](=O)N1CC2=CC(=NC(=C2[C@H]1CCO)C3=CC=CC(=C3)C4=CC=C(C=C4)F)C(=O)N5CCCCC5', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('OCCCCCCCCCC/C=C/CCCCCCCC', 'Long chain primary alcohol with "
               "20 carbons'), "
               "('O=C(O)/C(=C/CO)/CC=C1[C@](CC[C@@H]1C(C)C)(C/C=C(/C(=O)O)\\\\C)C', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('O=C(O)/C=C/C=C/[C@@H]1[C@@H](C(=C[C@H]2[C@H]1[C@H](C[C@@H](C2)O)C)C)C(=CCO)C', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('CC(C)(C)[S@@](=O)N1CC2=CC(=NC(=C2[C@H]1CCO)C3=CC(=CC=C3)C4=CCCC4)C(=O)N5CCN(CC5)C', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('CC(=O)N1[C@@H]([C@@H]([C@@H]1CO)C2=CC=C(C=C2)C3=CCCCC3)CNC(=O)CN(C)C', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('C[C@@H]1O[C@H](C[C@H](N)[C@@H]1O)O[C@H]1C[C@@](O)(Cc2cc3C(=O)c4cccc(O)c4C(=O)c3c(O)c12)C(=O)CO', "
               "'Long chain primary alcohol with 20 carbons'), "
               "('O[C@H](C(=C)CC/C=C(/CC/C=C(/CO)\\\\C)\\\\C)CC/C(=C/CO)/C', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('C1[C@@H]2[C@H]([C@@H](N2C(=O)CN1C(=O)NC3=CC=C(C=C3)F)CO)C4=CC=C(C=C4)C#CC5=CN=CC=C5', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('O1C(C(C2=C1C(OC)=CC(=C2)/C=C\\\\CO)CO)C3=CC(OC)=C(O)C=C3', "
               "'Long chain primary alcohol with 16 carbons'), "
               "('O=C1[C@]2(C(=C(CO)C3=C(C1)C(OC(C3)=O)(C)C)C[C@]4(C(=O)O[C@@H]5[C@H]4[C@H]2C(=O)[C@H](O5)C)C)C', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('CCCCCCCCCCCCCCC[C@@H](O)[C@H](CO)NC(=O)CCCCCCCCCCCCC', 'Long "
               "chain primary alcohol with 18 carbons'), "
               "('COC1=CC=CC(=C1)C2=CC3=C(C=C2)N[C@@H]([C@H]4[C@@H]3N(CC4)C(=O)CC5=CC=CC=C5)CO', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('O=C1N[C@@](OC)(C(=O)C2=CC=CC=C2)C([C@@]13OC(CO)=C(C3=O)C)O', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('CN1[C@H]2CN3C(=CC=C(C3=O)C4=CC(=CC=C4)C(=O)N(C)C)[C@@H]1[C@@H]([C@H]2CO)C(=O)NCC(F)(F)F', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('O=C1O[C@H](C2=C(O)C=CC(=C2)O)C=C1CC/C=C(\\\\CO)/C', 'Long "
               "chain primary alcohol with 14 carbons'), "
               "('O=C1C2=C(O[C@]3(CC=C([C@]3(C2)O)C(CCCC(CO)C)C)C)CC[C@@H]1O', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('COc1cc(C[C@@H]2[C@@H](Cc3ccc(OC)c(OC)c3)COC2=O)cc2[C@H](CO)[C@H](Oc12)c1ccc(O)c(OC)c1', "
               "'Long chain primary alcohol with 17 carbons'), "
               "('CCC(=O)N1C[C@@H]2[C@H]([C@@H](N2C(=O)C1)CO)C3=CC=C(C=C3)C#CC4=CN=CC=C4', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('COC1=CC=CC=C1C2=CC=C3[C@@H]4[C@@H]([C@H]([C@@H](N4CC5=CC=CC=C5)CN3C2=O)CO)C(=O)OC', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('O[C@H]1[C@@]2([C@H]([C@@H]3[C@@H]4[C@@](CC[C@H]4C(C)C)(C)CC[C@]3(C1)C)C2)CO', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('CCS(=O)(=O)N1CCCCN2[C@H](C1)[C@H]([C@H]2CO)C3=CC=C(C=C3)C4=CC=C(C=C4)C(=O)N(C)C', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('O1[C@]2([C@H]3[C@](C[C@]4(O)[C@H]([C@H](O)C[C@H]4C)C(=CC3)CO)(C)CC2)[C@@H](C)C[C@@H]1C=C(C)C', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('O[C@@H]([C@@H](NC(=O)CCC[C@@H](O)/C=C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)CO)\\\\C=C\\\\CCCCCCCCCCCCC', "
               "'Long chain primary alcohol with 18 carbons'), "
               "('[H][C@]12C[C@@H](OC(C)=O)C(CO)=C[C@H](O)[C@]1(C)CC[C@@]1(C)CCC(C(C)C)=C21', "
               "'Long chain primary alcohol with 15 carbons'), "
               "('C1CCC(=CC1)C2=CC=C(C=C2)[C@H]3[C@H]4CN(CC(=O)N4[C@@H]3CO)C(=O)CC5=CN=CC=C5', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('C1CCC(=CC1)C2=CC=C(C=C2)[C@@H]3[C@H]4CNCC(=O)N4[C@H]3CO', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('O=C(O)C=CC=CC1=C2C(=CC=C1C)C[C@@H](CO)C[C@H]2C', 'Long chain "
               "primary alcohol with 15 carbons'), "
               "('O=C1C=C2[C@]([C@H](CC[C@@H]2O)C)(C)[C@@H]3[C@H]1[C@]3(CO)C', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('CCC1=CN=C(S1)N2[C@H]([C@@H]([C@@H]2C#N)C3=CC=C(C=C3)C4=CCCCC4)CO', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('C1CCC(C1)(C#CC2=CC=C(C=C2)[C@H]3[C@H](N([C@@H]3C#N)C(=O)C4CC4)CO)O', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('C1[C@@H]2[C@@H]([C@@H](N2C(=O)CN1CC3=CC=C(C=C3)F)CO)C4=CC=C(C=C4)C5=CC=CC=C5', "
               "'Long chain primary alcohol with 13 carbons'), "
               "('CCCN1[C@@H]2[C@@H](CN3C2=CC=C(C3=O)C4=CC=C(C=C4)C(=O)N(C)C)[C@@H]([C@H]1C(=O)NCCOC)CO', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('CCCNC(=O)[C@@H]1[C@H]([C@@H]2CN3C(=CC=C(C3=O)C4=CC=CC=C4F)[C@@H]2N1CC5CC5)CO', "
               "'Long chain primary alcohol with 14 carbons'), "
               "('OCC12C(C(C(CC1)C)(CCC(CC(O)=O)C)C)CCC=C2CO', 'Long chain "
               "primary alcohol with 13 carbons')]\n"
               "False negatives: [('OC[*]', 'Chain too short (1 carbons)')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 100,
    'num_true_negatives': 27380,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.038461538461538464,
    'recall': 1.0,
    'f1': 0.07407407407407407,
    'accuracy': 0.9963615194294863}


"""
Classifies: CHEBI:24662 hydroxy-amino acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxy_amino_acid(smiles: str):
    """
    Determines if a molecule is a hydroxy-amino acid (non-proteinogenic alpha-amino acid with one or more hydroxy groups).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxy-amino acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"

    # Check for amine group (primary, secondary or tertiary)
    amine_pattern = Chem.MolFromSmarts('[NX3;H2,H1,H0]')
    if not mol.HasSubstructMatch(amine_pattern):
        return False, "No amine group found"

    # Check for hydroxy group
    hydroxy_pattern = Chem.MolFromSmarts('[OH]')
    hydroxy_matches = mol.GetSubstructMatches(hydroxy_pattern)
    
    # Don't count the OH from carboxylic acid
    carboxylic_matches = mol.GetSubstructMatches(carboxylic_pattern)
    carboxylic_o_indices = set(match[1] for match in carboxylic_matches)  # Get O indices from COOH
    
    true_hydroxy_matches = []
    for match in hydroxy_matches:
        if match[0] not in carboxylic_o_indices:
            true_hydroxy_matches.append(match)
            
    if not true_hydroxy_matches:
        return False, "No hydroxy groups found (excluding carboxylic acid OH)"

    # Check if amine and carboxylic acid are on same carbon (alpha position)
    alpha_amino_acid_pattern = Chem.MolFromSmarts('[NX3;H2,H1,H0][CH1]C(=O)[OH]')
    if not mol.HasSubstructMatch(alpha_amino_acid_pattern):
        return False, "Amine and carboxylic acid groups not in alpha position"

    # Count number of hydroxy groups (excluding carboxylic acid OH)
    num_hydroxy = len(true_hydroxy_matches)

    return True, f"Hydroxy-amino acid with {num_hydroxy} hydroxy group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24662',
                          'name': 'hydroxy-amino acid',
                          'definition': 'A non-proteinogenic alpha-amino acid '
                                        'bearing one or more hydroxy groups at '
                                        'unspecified positions.',
                          'parents': ['CHEBI:83925']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Invariant Violation\n'
               '\t\n'
               '\tViolation occurred on line 341 in file '
               'Code/GraphMol/Matrices.cpp\n'
               '\tFailed Expression: aid1 != aid2\n'
               '\tRDKIT: 2024.03.6\n'
               '\tBOOST: 1_85\n',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 12,
    'num_false_positives': 100,
    'num_true_negatives': 1437,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.10714285714285714,
    'recall': 1.0,
    'f1': 0.19354838709677416,
    'accuracy': 0.9354422207876049}


"""
Classifies: CHEBI:23899 icosanoid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_icosanoid(smiles: str):
    """
    Determines if a molecule is an icosanoid based on its SMILES string.
    Icosanoids are signaling molecules derived from C20 fatty acids.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an icosanoid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Check carbon count - should be around 20 carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 15 or carbon_count > 25:
        return False, f"Carbon count {carbon_count} outside typical range for icosanoids (15-25)"

    # Define and check SMARTS patterns
    patterns = {
        'carboxyl': '[CX3](=O)[OX2H1]',
        'double_bond': '[CX3]=[CX3]',
        'hydroxy': '[OX2H]',
        'carbonyl': '[CX3](=O)[CX4]',
        'carbon_chain': 'CCCCCC'
    }

    features = []
    matched_patterns = {}

    for name, pattern in patterns.items():
        pat = Chem.MolFromSmarts(pattern)
        if pat is not None and mol.HasSubstructMatch(pat):
            matched_patterns[name] = True
            if name != 'carbon_chain':  # Don't include carbon chain in features list
                features.append(name.replace('_', ' '))

    # Must have carboxylic acid group
    if not matched_patterns.get('carboxyl'):
        return False, "No carboxylic acid group found"

    # Must have carbon chain
    if not matched_patterns.get('carbon_chain'):
        return False, "No long carbon chain found"

    # Should have at least one other typical feature
    if len(features) < 2:  # carboxyl is already one feature
        return False, "Insufficient typical icosanoid structural features"

    feature_description = []
    if matched_patterns.get('double_bond'):
        feature_description.append("unsaturated bonds")
    if matched_patterns.get('hydroxy'):
        feature_description.append("hydroxyl groups")
    if matched_patterns.get('carbonyl'):
        feature_description.append("carbonyl groups")
    if matched_patterns.get('carboxyl'):
        feature_description.append("carboxylic acid")

    return True, f"Icosanoid containing {', '.join(feature_description)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23899',
                          'name': 'icosanoid',
                          'definition': 'Any member of the group of signalling '
                                        'molecules arising from oxidation of '
                                        'the three C20 essential fatty acids '
                                        '(EFAs) icosapentaenoic acid (EPA), '
                                        'arachidonic acid (AA) and '
                                        'dihomo-gamma-linolenic acid (DGLA).',
                          'parents': ['CHEBI:61697']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.HasSubstructMatch(Mol, NoneType)\n'
               'did not match C++ signature:\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, RDKit::SubstructMatchParameters params=True)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'RDKit::SubstructMatchParameters params)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, bool recursionPossible=True, bool useChirality=False, '
               'bool useQueryQueryMatches=False)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'bool recursionPossible=True, bool useChirality=False, bool '
               'useQueryQueryMatches=False)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 55,
    'num_false_positives': 100,
    'num_true_negatives': 2412,
    'num_false_negatives': 37,
    'num_negatives': None,
    'precision': 0.3548387096774194,
    'recall': 0.5978260869565217,
    'f1': 0.44534412955465585,
    'accuracy': 0.9473886328725039}


"""
Classifies: CHEBI:22715 benzimidazoles
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_benzimidazoles(smiles: str):
    """
    Determines if a molecule is a benzimidazole (benzene ring fused to an imidazole ring).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a benzimidazole, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define SMARTS pattern for benzimidazole core structure
    # [nH]1cnc2ccccc12 or n1c[nH]c2ccccc12
    benzimidazole_pattern = Chem.MolFromSmarts('[nH]1cnc2ccccc12')
    benzimidazole_pattern2 = Chem.MolFromSmarts('n1c[nH]c2ccccc12')
    
    # Also check for N-substituted variants
    n_subst_pattern = Chem.MolFromSmarts('n1cnc2ccccc12')

    matches = mol.GetSubstructMatches(benzimidazole_pattern)
    matches2 = mol.GetSubstructMatches(benzimidazole_pattern2)
    n_subst_matches = mol.GetSubstructMatches(n_subst_pattern)

    if matches or matches2 or n_subst_matches:
        # Find substituents
        substituents = []
        if matches or matches2:
            core_atoms = set(matches[0] if matches else matches2[0])
        else:
            core_atoms = set(n_subst_matches[0])
            
        for atom_idx in core_atoms:
            atom = mol.GetAtomWithIdx(atom_idx)
            for neighbor in atom.GetNeighbors():
                if neighbor.GetIdx() not in core_atoms:
                    substituents.append(neighbor.GetSymbol())

        if len(substituents) > 0:
            return True, f"Substituted benzimidazole with substituents at: {', '.join(set(substituents))}"
        else:
            return True, "Unsubstituted benzimidazole"
            
    return False, "No benzimidazole core structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22715',
                          'name': 'benzimidazoles',
                          'definition': 'An organic heterocyclic compound '
                                        'containing a benzene ring fused to an '
                                        'imidazole ring.',
                          'parents': ['CHEBI:27171', 'CHEBI:38101']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 39,
    'num_false_positives': 100,
    'num_true_negatives': 73271,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.2805755395683453,
    'recall': 0.975,
    'f1': 0.43575418994413406,
    'accuracy': 0.9986241843865361}


"""
Classifies: CHEBI:23247 cinnamamides
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import IPythonConsole

def is_cinnamamides(smiles: str):
    """
    Determines if a molecule is a cinnamamide derivative.
    Cinnamamides contain:
    - An amide group (-C(=O)N-)
    - A double bond conjugated with the amide carbonyl
    - A phenyl ring conjugated with the double bond
    
    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a cinnamamide, False otherwise
        str: Reason for classification
    """
    
    # Check valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for amide group
    amide_pattern = Chem.MolFromSmarts('[CX3](=[OX1])[NX3]')
    if not mol.HasSubstructMatch(amide_pattern):
        return False, "No amide group found"

    # Look for conjugated double bond and phenyl pattern
    cinnamamide_pattern = Chem.MolFromSmarts('[CX3](=[OX1])[NX3]-,=[#6]-[#6]=[#6]-[cX3]1[cX3][cX3][cX3][cX3][cX3]1')
    
    if not mol.HasSubstructMatch(cinnamamide_pattern):
        return False, "Missing required cinnamamide substructure (conjugated double bond and phenyl ring)"

    # Get the matches
    matches = mol.GetSubstructMatches(cinnamamide_pattern)
    
    # Check stereochemistry of double bond if explicit
    has_explicit_stereochem = False
    for match in matches:
        bond_atoms = mol.GetBondBetweenAtoms(match[3], match[4])
        if bond_atoms.GetStereo() in [Chem.BondStereo.STEREOE, Chem.BondStereo.STEREOZ]:
            has_explicit_stereochem = True
            
    stereo_msg = " (with explicit double bond stereochemistry)" if has_explicit_stereochem else " (no explicit double bond stereochemistry)"

    # Look for substituents
    base_cinnamamide = Chem.MolFromSmiles("NC(=O)C=Cc1ccccc1")
    if Chem.MolToSmiles(mol) == Chem.MolToSmiles(base_cinnamamide):
        return True, "Unsubstituted cinnamamide" + stereo_msg
    else:
        return True, "Substituted cinnamamide derivative" + stereo_msg


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23247',
                          'name': 'cinnamamides',
                          'definition': 'An enamide which is cinnamamide or a '
                                        'derivative of cinnamamide obtained by '
                                        'replacement of one or more of its '
                                        'hydrogens.',
                          'parents': ['CHEBI:51751']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 146702,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9992643552891493}


"""
Classifies: CHEBI:26191 polyol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_polyol(smiles: str):
    """
    Determines if a molecule is a polyol (contains 2 or more hydroxy groups).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a polyol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Get all OH groups
    oh_pattern = Chem.MolFromSmarts('[OH]')
    oh_matches = mol.GetSubstructMatches(oh_pattern)
    
    num_oh = len(oh_matches)
    
    if num_oh < 2:
        return False, f"Contains only {num_oh} hydroxy group(s), minimum 2 required"
    
    return True, f"Contains {num_oh} hydroxy groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26191',
                          'name': 'polyol',
                          'definition': 'A compound that contains two or more '
                                        'hydroxy groups.',
                          'parents': ['CHEBI:33822']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 148,
    'num_false_positives': 100,
    'num_true_negatives': 122,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.5967741935483871,
    'recall': 1.0,
    'f1': 0.7474747474747475,
    'accuracy': 0.7297297297297297}